--============================================================================
-- Design units : general
-- File name    : detectClock.vhd
--
-- Description  : detects clock running on normal (not clock) in pin
--		  detClk1: a short glitch has no effect on the result
--		  detClk2: a short glitch produces a one clock cycle output
--		  selClk:  choose a running clock (if possible
-- Uses         : None
--
-- Author       : Klaus Ruzicka, Zurich, Switzerland
--
--============================================================================
-- Revision list
-- Version  Author  Date      Changes
--
-- 1.0      Rkl     07/01/99  New design
--============================================================================

--============================================================================
library IEEE;
use IEEE.std_logic_1164.all;

entity detClk1 is
-- detect clock, short glitches does not affect the result
-- use constant edge to define active edge for FFs
  port( clk	: in  std_logic;	-- target clock domain
	D	: in  std_logic;	-- asynchronous input
	Q	: out std_logic);	-- synchronous clock detect 
					-- clk@D: 1= running, 0=stopped
end detClk1;

architecture rtl of detClk1 is
  constant edge      : std_logic := '1'; -- output active 1=rising 0=falling edge
  signal Q1,Q2,Q3,Q4 : std_logic := '0'; -- internal signals
  begin

    -- catch clk on D in every case, even when Td<Tclk 
    FF1: process (D, clk)	-- FF with asynchronous set and synchronous reset
    begin
      if D='1' then
        Q1<='1';
      else
        if clk'event and clk=edge then
          Q1<='0';
        end if;
      end if;
    end process;
    FF2: process (D, clk)	-- FF with asynchronous set and synchronous reset
    begin
      if D='0' then
        Q2<='1';
      else
        if clk'event and clk=edge then
          Q2<='0';
        end if;
      end if;
    end process;  
    Q3<=Q1 and Q2 and not clk;

    FF3: process (clk)
    begin
      if clk'event and clk=edge then
        Q4<=Q3;
        Q<=Q4 and Q3;	-- FF to suppress short (<1Tclk) glitches
      end if;
    end process;
end rtl;
-- ===========================================================================
library IEEE;
use IEEE.std_logic_1164.all;

entity detClk2 is
-- detect clock, a short glitch produces a one clock cycle output
-- use constant edge to define active edge for FFs
  port( clk	: in  std_logic;	-- target clock domain
	D	: in  std_logic;	-- asynchronous clock signal
	Q	: out std_logic);	-- synchronous clock detect 
					-- clk@D: 1= running, 0=stopped
end detClk2;

architecture rtl of detClk2 is
  constant edge      : std_logic := '1'; -- output active 1=rising 0=falling edge
  signal Q1,Q2,Q3,Q4 : std_logic := '0'; -- internal signals
  begin

    -- catch clk on D in every case, even when Td<Tclk 
    FF1: process (D, clk)	-- FF with asynchronous set and synchronous reset
    begin
      if D='1' then
        Q1<='1';
      else
        if clk'event and clk=edge then
          Q1<='0';
        end if;
      end if;
    end process;
    FF2: process (D, clk)	-- FF with asynchronous set and synchronous reset
    begin
      if D='0' then
        Q2<='1';
      else
        if clk'event and clk=edge then
          Q2<='0';
        end if;
      end if;
    end process;  
    Q3<=Q1 and Q2 and not clk;

    FF3: process (clk)
    begin
      if clk'event and clk=edge then
        Q4<=Q3;
      end if;
    end process;
    Q<=Q4 or Q3;	
end rtl;
-- ===========================================================================
library IEEE;
use IEEE.std_logic_1164.all;

entity selClk is
-- choose a running clock (if possible)
-- select clock (or data signal related to clock) with
--	  clkOut <= (cklP and sel) or (clkR and not sel);
-- glitches are no problem as long as one clock is running
  port( clk	  : in  std_logic;	-- target clock domain
	clkP,clkR : in  std_logic;	-- asynchronous clock signals
        sel       : out std_logic);	-- which clock? 0=prim, 1=red
end selClk;

architecture rtl of selClk is
  component detClk2 
    port( clk	: in  std_logic;	-- target clock domain
	D	: in  std_logic;	-- asynchronous clock signal
	Q	: out std_logic);	-- synchronous clock detect 
					-- clk@D: 1= running, 0=stopped
  end component;
  signal runPi, runRi : std_logic := '0';

  begin
  P:detClk2 port map (clk, clkP, runPi);
  R:detClk2 port map (clk, clkR, runRi);

  process(runPi, runRi)
  begin
    if (runPi and runRi)='0' then
      if runPi = '1' then
        sel <= '0';
      else 
        sel <= '1'; 
      end if;
    end if;
  end process;
end rtl;
-- ===========================================================================
-- end of file
-- ===========================================================================
